/*
 * Tray Up Time
 *
 * PDH performance counter query
 *
 * Copyright (c) 2009-2010 by Joergen Ibsen / Jibz
 * All Rights Reserved
 *
 * http://www.ibsensoftware.com/
 *
 * This software is provided 'as-is', without any express
 * or implied warranty.  In no event will the authors be
 * held liable for any damages arising from the use of
 * this software.
 *
 * Permission is granted to anyone to use this software
 * for any purpose, including commercial applications,
 * and to alter it and redistribute it freely, subject to
 * the following restrictions:
 *
 * 1. The origin of this software must not be
 *    misrepresented; you must not claim that you
 *    wrote the original software. If you use this
 *    software in a product, an acknowledgment in
 *    the product documentation would be appreciated
 *    but is not required.
 *
 * 2. Altered source versions must be plainly marked
 *    as such, and must not be misrepresented as
 *    being the original software.
 *
 * 3. This notice may not be removed or altered from
 *    any source distribution.
 */

#include "suptime.h"

#ifdef _MSC_VER
#pragma comment(lib,"pdh.lib")
#endif

BOOL sut_open(SUPTIME *sut)
{
    PDH_STATUS status;
    wchar_t szCounterPath[PDH_MAX_COUNTER_PATH + 1];

    DWORD cbUsed, cbLeft, cbSize;

    if (sut == NULL) return FALSE;

	/* construct path to localized system up time counter */
    cbUsed = 0;
    cbLeft = sizeof(szCounterPath) / sizeof(szCounterPath[0]);

    szCounterPath[cbUsed] = '\\';
    cbUsed++;
    cbLeft--;

    cbSize = cbLeft;
    PdhLookupPerfNameByIndex(NULL, 2, &szCounterPath[cbUsed], &cbSize);
    cbSize--;
    cbUsed += cbSize;
    cbLeft -= cbSize;

    szCounterPath[cbUsed] = '\\';
    cbUsed++;
    cbLeft--;

    cbSize = cbLeft;
    PdhLookupPerfNameByIndex(NULL, 674, &szCounterPath[cbUsed], &cbSize);

	/* check path is valid */
    if (PdhValidatePath(szCounterPath) != ERROR_SUCCESS) return FALSE;

    /* create query */
    status = PdhOpenQuery(NULL, 0, &sut->hQuery);

    if (status != ERROR_SUCCESS) return FALSE;

    /* add counter to query */
    status = PdhAddCounter(sut->hQuery, szCounterPath, 0, &sut->hCounter);

    if (status != ERROR_SUCCESS) { PdhCloseQuery(sut->hQuery); return FALSE; }

    return TRUE;
}

BOOL sut_get(SUPTIME *sut, LONGLONG *v)
{
    PDH_STATUS status;
    PDH_FMT_COUNTERVALUE Value;

    if (sut == NULL || v == NULL) return FALSE;

    status = PdhCollectQueryData(sut->hQuery);

    if (status != ERROR_SUCCESS) return FALSE;

    status = PdhGetFormattedCounterValue(sut->hCounter, PDH_FMT_LARGE, NULL, &Value);

    if (status != ERROR_SUCCESS) return FALSE;

    if (Value.CStatus != PDH_CSTATUS_NEW_DATA && Value.CStatus != PDH_CSTATUS_VALID_DATA) return FALSE;

    if (v) *v = Value.largeValue;

    return TRUE;
}

BOOL sut_close(SUPTIME *sut)
{
    if (sut == NULL) return FALSE;

    /* close query */
    return PdhCloseQuery(sut->hQuery) == ERROR_SUCCESS;
}
