/*
 * Tray Up Time
 *
 * PDH performance counter query
 *
 * Copyright (c) 2009-2010 by Joergen Ibsen / Jibz
 * All Rights Reserved
 *
 * http://www.ibsensoftware.com/
 *
 * This software is provided 'as-is', without any express
 * or implied warranty.  In no event will the authors be
 * held liable for any damages arising from the use of
 * this software.
 *
 * Permission is granted to anyone to use this software
 * for any purpose, including commercial applications,
 * and to alter it and redistribute it freely, subject to
 * the following restrictions:
 *
 * 1. The origin of this software must not be
 *    misrepresented; you must not claim that you
 *    wrote the original software. If you use this
 *    software in a product, an acknowledgment in
 *    the product documentation would be appreciated
 *    but is not required.
 *
 * 2. Altered source versions must be plainly marked
 *    as such, and must not be misrepresented as
 *    being the original software.
 *
 * 3. This notice may not be removed or altered from
 *    any source distribution.
 */

#ifndef SUPTIME_H_INCLUDED
#define SUPTIME_H_INCLUDED

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <pdh.h>
#include <pdhmsg.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef PDH_MAX_COUNTER_PATH
#define PDH_MAX_COUNTER_PATH 2048
#endif

typedef struct {
    PDH_HQUERY hQuery;
    PDH_HCOUNTER hCounter;
} SUPTIME;

/** Create a performance counter query for the System Up Time counter.
 *  @param sut - pointer to SUPTIME structure.
 *  @return @c TRUE on success, @c FALSE otherwise.
 */
BOOL sut_open(SUPTIME *sut);

/** Collect data from the specified query.
 *  @param sut - pointer to SUPTIME structure.
 *  @param v   - pointer to variable receiving the data.
 *  @return @c TRUE on success, @c FALSE otherwise.
 */
BOOL sut_get(SUPTIME *sut, LONGLONG *v);

/** Close query.
 *  @param sut - pointer to SUPTIME structure.
 *  @return @c TRUE on success, @c FALSE otherwise.
 */
BOOL sut_close(SUPTIME *sut);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* SUPTIME_H_INCLUDED */
